﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using AIOUSBNet;  // the namespace exposes the AIOUSB Class interface 

// You must also add a reference to the AIOUSBnet.dll in the project settings

namespace Sample1
{
    public partial class Form1 : Form
    {
        // One and only Device Index:
        public UInt32 DeviceIndex;
        public UInt32 numPorts = 2;

        // Interval Timer fou GUI update:
        // (Note: A Windows Forms timer designed for single threaded environments not a System Timer 55 ms min res)
        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();

        // Array of controls for GUI updates:
        public CheckBox[] checkStateArray = new CheckBox[16];
        public Button[] toggleBtnArray = new Button[2];

        // For bitmasks:
        byte[] OutMask = new byte[] { 0 };
        byte[] Data = new byte[] { 0, 0 };

        // State flags:
        byte bTristateAll = 0;  // 0 = false all bits not tristated
        public bool[] bModeOut = new bool[2];
        
        public Form1()
        {
            InitializeComponent();

            // Array of the toggle mode buttons:
            toggleBtnArray[0] = btnToggle0;
            toggleBtnArray[1] = btnToggle1;

            // Array of the LED style checkboxes:
            checkStateArray[0] = checkBoxA0_1;
            checkStateArray[1] = checkBoxA0_2;
            checkStateArray[2] = checkBoxA0_3;
            checkStateArray[3] = checkBoxA0_4;

            checkStateArray[4] = checkBoxB0_9;
            checkStateArray[5] = checkBoxB0_10;
            checkStateArray[6] = checkBoxB0_11;
            checkStateArray[7] = checkBoxB0_12;
        }
        
        private void Form1_Load(object sender, EventArgs e)
        {
            UInt32 Status;

            FormDetect DetectSubForm = new FormDetect(this);
            DetectSubForm.ShowDialog();

            // Init values:
            for (int i = 0; i < numPorts; i++)
            {
                Data[i] = 0xFF;
                bModeOut[i] = false;  // Default power up not Output hence Input Mode
            }

            Status = AIOUSB.DIO_Configure(DeviceIndex, bTristateAll, OutMask, Data);
            
            // Add the event and the event handler for the method that will 
            // process the timer event to the timer:
            myTimer.Tick += new EventHandler(TimerEventProcessor);

            // Set the timer interval in miliseconds and start 55ms min resolution:
            myTimer.Interval = 100;
            myTimer.Start();
        }
        
        private void TimerEventProcessor(Object myObject, EventArgs myEventArgs)
        {
            // This is the method that runs when the timer event is raised:
            UpdateGUIState();
        }
        
        private void UpdateGUIState()
        {
            // Get data from board update display states:
            Data[0] = 0;
            Data[1] = 0;
            AIOUSB.DIO_ReadAll(DeviceIndex, Data);

            // Test each bit and set check state:
            for (int i = 0; i < 4; i++)
            {
                if( (Data[0] & ( 1 << (i) )) != 0 ) 
                    checkStateArray[i].Checked = true; 
                else
                    checkStateArray[i].Checked = false;

                if ((Data[1] & (1 << (i))) != 0) 
                    checkStateArray[i+4].Checked = true; 
                else
                    checkStateArray[i+4].Checked = false; 
            }
        }
  
        private void btnToggle_Click(object sender, EventArgs e)
        {
            // Button Clicks to toggle the input output mode and display states:

            // Need to stop the timer change board and GUI:
            myTimer.Stop(); 

            Int32 iBit, iByte;

            //Get button sender Tag (index)
            UInt32 iTagindex = 0;
            Button btnSender = sender as Button;
            if (btnSender != null)
            {
                iTagindex = Convert.ToUInt32(btnSender.Tag);
            }

            // Toggle local mode value and button display states:
            if (bModeOut[iTagindex])
            {
                bModeOut[iTagindex] = false;
                btnSender.Text = "Toggle to Output Mode";
                //label2.Text = "Output State:";
            }
            else
            {
                bModeOut[iTagindex] = true;
                btnSender.Text = "Toggle to Input Mode";
                //label2.Text = "Input State:";
            }

            // Recreate new Outmask for Configure:
            for (iByte = 0; iByte < OutMask.Length; iByte++)
                OutMask[iByte] = 0
            ;
            for (iBit = 0; iBit < numPorts; iBit++)
            {
                if (bModeOut[iBit] == true)  // if output mode
                {
                    iByte = iBit >> 3;
                    OutMask[iByte] = (byte)(OutMask[iByte] | (byte)(1 << (iBit & 7)));
                }
            }

            AIOUSB.DIO_Configure(DeviceIndex, bTristateAll, OutMask, Data);

            // Restart timer thread:
            myTimer.Enabled = true;
        }

        private void btnDevice_Click(object sender, EventArgs e)
        {
            // Launch connect device dialog box
            // Switch between multiple devices or reconnect

            // New parent aware subform:
            FormDetect DetectSubForm = new FormDetect(this);
            DetectSubForm.ShowDialog();
        }

        private void checkBox__Click(object sender, EventArgs e)
        {
            // Need to stop the timer change board and GUI:
            myTimer.Stop();

            bool bHighLow = false;
            UInt32 iTagindex = 0;

            CheckBox chkSender = sender as CheckBox;
            if (chkSender != null)
            {
                iTagindex = Convert.ToUInt32(chkSender.Tag);
            }
            bHighLow = chkSender.Checked;

            Int32 i;
            Int32 bitValue = 0x0000;
            byte bByte = 1;

            i = (Int32)iTagindex;  // cast ok

            bitValue = (Data[i / 8]) & (1 << (i % 8)); // get single bit
            if (bitValue != 0)
                bByte = 0;

            AIOUSB.DIO_Write1(DeviceIndex, (UInt32)i, bByte);  // byte used like a bool

            // restart timer and GUI will update:
            myTimer.Enabled = true;
        }

    }
}
